SystemOrganization addCategory: #'Khami-Core'!
SystemOrganization addCategory: #'Khami-Core-Locations'!
SystemOrganization addCategory: #'Khami-Core-Support'!

GRObject subclass: #KmGame
	instanceVariableNames: 'tileSet players board currentTile lastTile currentPlayer owner startDate colours'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmGame class methodsFor: 'instance creation' stamp: 'StuartHerring 3/14/2011 10:06'!
testGame
	^self tileSet: KmTileSet testSet! !

!KmGame class methodsFor: 'instance creation' stamp: 'StuartHerring 3/14/2011 10:06'!
tileSet: aTileSet 
	^self basicNew
		initializeWithTileSet: aTileSet;
		yourself! !

!KmGame methodsFor: 'accessing' stamp: 'StuartHerring 7/6/2011 20:39'!
addPlayer: aPlayer
	startDate ifNotNil: [KmInvalidGameState signal: 'Cannot add players to a game in progress'].
	owner ifNil: [ owner := aPlayer user].
	aPlayer colour: (colours remove: colours atRandom).
	players add: aPlayer! !

!KmGame methodsFor: 'accessing' stamp: 'StuartHerring 1/12/2011 19:33'!
board
	^board! !

!KmGame methodsFor: 'actions' stamp: 'StuartHerring 5/12/2011 11:21'!
calculateFinalScore
	self territories do: [ :t | t calculateFinalScore].! !

!KmGame methodsFor: 'actions' stamp: 'StuartHerring 5/14/2011 22:01'!
calculateScore
	self territories do: [ :t | t calculateScore].! !

!KmGame methodsFor: 'initialization' stamp: 'StuartHerring 5/10/2011 21:51'!
currentPlayer
	^currentPlayer! !

!KmGame methodsFor: 'accessing' stamp: 'StuartHerring 1/16/2011 20:19'!
currentTile
	^currentTile! !

!KmGame methodsFor: 'actions' stamp: 'StuartHerring 5/9/2011 21:53'!
discardTile
	self validMoveAvailable ifTrue: [ KmInvalidGameState signal: 'Cannot discard a tile when a valid move is available'].
	currentTile := nil! !

!KmGame methodsFor: 'initialization' stamp: 'StuartHerring 7/6/2011 20:38'!
initializeWithTileSet: aSet
	self initialize.
	tileSet := aSet.
	board := KmGameBoard startingTile: aSet takeTile.
	currentTile := nil.
	players := OrderedCollection  new.
	colours := {Color blue. Color green. Color red. Color orange. Color magenta. Color cyan} asSet.! !

!KmGame methodsFor: 'testing' stamp: 'StuartHerring 6/12/2011 15:34'!
isComplete
	"Returns true if there are no more tiles to be found"
	^tileSet size = 0! !

!KmGame methodsFor: 'testing' stamp: 'StuartHerring 7/6/2011 22:26'!
isInProgress
	^startDate notNil and: [self isComplete not]! !

!KmGame methodsFor: 'testing' stamp: 'StuartHerring 6/22/2011 21:31'!
isPending
	^startDate isNil! !

!KmGame methodsFor: 'accessing' stamp: 'StuartHerring 5/10/2011 21:27'!
lastTile
	^lastTile! !

!KmGame methodsFor: 'actions' stamp: 'StuartHerring 5/10/2011 21:54'!
nextPlayer
	^currentPlayer := 
		(currentPlayer isNil or: [currentPlayer = players last])
			ifTrue: [ players first]
			ifFalse: [ players after: currentPlayer ]! !

!KmGame methodsFor: 'initialization' stamp: 'StuartHerring 6/22/2011 21:10'!
owner
	^owner! !

!KmGame methodsFor: 'actions' stamp: 'StuartHerring 5/14/2011 21:32'!
placeCurrentTileAtRow: row andColumn: column
	currentTile 
		ifNil: [KmInvalidGameState signal: 'No current tile to be placed'].
	(board canPlaceTile: currentTile at:  row at:  column) 
		ifFalse: [KmInvalidMove signal].
	self updateHutCountsForTilePlacedAtRow: row andColumn: column.
	board at: row at: column put: currentTile.
	lastTile := currentTile.
	currentTile := nil.! !

!KmGame methodsFor: 'accessing' stamp: 'StuartHerring 5/9/2011 20:49'!
players
	^players! !

!KmGame methodsFor: 'actions' stamp: 'StuartHerring 7/4/2011 20:53'!
start
	startDate ifNil: [
		players ifEmpty: [KmInvalidGameState signal: 'Cannot start a game without players'].  
		startDate := DateAndTime now.
		currentPlayer := players atRandom	]! !

!KmGame methodsFor: 'actions' stamp: 'StuartHerring 6/22/2011 21:51'!
startTime
	^startDate! !

!KmGame methodsFor: 'actions' stamp: 'StuartHerring 4/10/2011 16:04'!
takeTile
	^currentTile := currentTile ifNil: [tileSet takeTile].! !

!KmGame methodsFor: 'accessing' stamp: 'StuartHerring 1/30/2011 22:55'!
territories
	| result |
	result := Set new.
	board do: [ :tile | result addAll: tile territories].
	^result! !

!KmGame methodsFor: 'accessing' stamp: 'StuartHerring 3/14/2011 11:56'!
tileSet
	^tileSet! !

!KmGame methodsFor: 'actions' stamp: 'StuartHerring 5/14/2011 21:52'!
updateHutCountsForTilePlacedAtRow: row andColumn: col
	(board surroundingTilesAt: row at: col) do: [ :t | 
		t addSurroundingTile.
		currentTile addSurroundingTile]! !

!KmGame methodsFor: 'testing' stamp: 'StuartHerring 4/10/2011 13:39'!
validMoveAvailable
	^currentTile ifNil: [false]
		ifNotNil: [ | tile | 
			tile := currentTile veryDeepCopy.
			(1 to: board rows) anySatisfy: [ :r | 
				(1 to: board columns) anySatisfy: [ :c | 
					(1 to: 4) anySatisfy: [ :i | 
						tile rotate. 
						(board canPlaceTile: tile at:  r at: c)]]]].
! !

GRObject subclass: #KmGameBoard
	instanceVariableNames: 'startingTile grid'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmGameBoard class methodsFor: 'instance creation' stamp: 'StuartHerring 3/7/2011 20:23'!
startingTile: aTile
	^self basicNew
		initializeWithStartingTile: aTile;
		yourself.! !

!KmGameBoard methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 21:06'!
at: row at: column
	(row < 1 or: [column < 1 or: [row > grid rows or: [column > grid columns]]]) ifTrue: [^nil].
	^grid at: row at: column! !

!KmGameBoard methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 21:06'!
at: row at: column put: aTile
	grid at: row at: column put: aTile.
	self connectTilesAt: row at: column.
	(row = 1) ifTrue: [ grid prependRow].
	(row = grid rows) ifTrue: [grid addRow].
	(column = 1) ifTrue: [ grid prependColumn ].
	(column = grid columns) ifTrue: [ grid addColumn].! !

!KmGameBoard methodsFor: 'testing' stamp: 'StuartHerring 5/14/2011 21:06'!
canPlaceTile: aTile at: row at: column
	(grid at: row at: column) ifNotNil: [^false].
	((grid adjacentItemsAt: row at: column) noneSatisfy: [ :item | item notNil])
		ifTrue: [ ^false ].
	^KmLocation edges 
		allSatisfy: [ :side | self canPlaceTile: aTile toThe: side of: row and: column]
! !

!KmGameBoard methodsFor: 'testing' stamp: 'StuartHerring 5/14/2011 21:06'!
canPlaceTile: aTile toThe: aSide of: row and: column
	| r c |
	(aSide = Left) ifTrue: [ c := column - 1 ] 
		ifFalse: [ (aSide = Right) ifTrue: [ c := column + 1] ifFalse: [ c := column]].
	(aSide = Top) ifTrue: [ r := row - 1 ] 
		ifFalse: [ (aSide = Bottom) ifTrue: [ r := row + 1] ifFalse: [ r := row]].
	((r < 1) or: [c < 1]) ifTrue: [^true].
	((r > grid rows) or: [c > grid columns]) ifTrue: [^true].
	(grid at: r at: c) ifNil: [^true].
	^(grid at: r at: c) allowsTile: aTile at: aSide opposite! !

!KmGameBoard methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 21:06'!
columns
	^grid columns! !

!KmGameBoard methodsFor: 'actions' stamp: 'StuartHerring 5/14/2011 21:06'!
connectTilesAt: row at: column
	| tile |
	tile := grid at: row at: column.
	tile tileAt: Top put: (self at: row - 1at: column).
	tile tileAt: Bottom put: (self at: row + 1at: column).
	tile tileAt: Left put: (self at: row at: column - 1).
	tile tileAt: Right put: (self at: row at: column + 1).
	! !

!KmGameBoard methodsFor: 'enumerating' stamp: 'StuartHerring 5/14/2011 21:06'!
do: aBlock
	^grid do: [ :tile | tile ifNotNil: [ aBlock value: tile]]! !

!KmGameBoard methodsFor: 'initialization' stamp: 'StuartHerring 5/14/2011 21:06'!
initializeWithStartingTile: aTile
	self initialize.
	startingTile := aTile.
	grid := KmGrid ofSize: 3 by: 3.
	grid at: 2 at: 2 put: startingTile! !

!KmGameBoard methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 21:06'!
rows
	^grid rows! !

!KmGameBoard methodsFor: 'accessing' stamp: 'StuartHerring 1/12/2011 19:46'!
startingTile
	^startingTile! !

!KmGameBoard methodsFor: 'actions' stamp: 'StuartHerring 5/14/2011 21:41'!
surroundingTilesAt: row at:  column
	^(grid surroundingItemsAt: row at: column) reject: [ :t | t isNil]  ! !

GRObject subclass: #KmGrid
	instanceVariableNames: 'rows cols'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Support'!

!KmGrid class methodsFor: 'instance creation' stamp: 'StuartHerring 1/10/2011 14:30'!
ofSize: rows by: columns
	^self new
		addColumns: columns;
		addRows: rows;
		yourself.! !

!KmGrid methodsFor: 'actions' stamp: 'StuartHerring 1/17/2011 11:18'!
addColumn
	self addColumns: 1! !

!KmGrid methodsFor: 'actions' stamp: 'StuartHerring 1/10/2011 14:28'!
addColumns: aNumber
	rows do:  [:row |
		aNumber timesRepeat: [ row add: nil ]].
	cols := cols + aNumber.
		! !

!KmGrid methodsFor: 'actions' stamp: 'StuartHerring 1/17/2011 11:20'!
addRow
	self addRows: 1! !

!KmGrid methodsFor: 'actions' stamp: 'StuartHerring 1/10/2011 14:07'!
addRows: aNumber
	aNumber timesRepeat:  [
		rows add: (OrderedCollection ofSize: cols)]! !

!KmGrid methodsFor: 'accessing' stamp: 'StuartHerring 1/18/2011 19:22'!
adjacentItemsAt: row at: column
	| return |
	return := OrderedCollection new.
	(row > 1) ifTrue: [ return add: (self at: row -1 at: column)].
	(column < cols) ifTrue: [ return add: (self at: row at: column + 1)].
	(row < rows size) ifTrue: [ return add: (self at: row  +1 at: column)].
	(column > 1) ifTrue: [ return add: (self at: row at: column - 1)].
	^return! !

!KmGrid methodsFor: 'accessing' stamp: 'StuartHerring 1/12/2011 09:44'!
at: row at: column
	^(rows at: row) at: column ! !

!KmGrid methodsFor: 'accessing' stamp: 'StuartHerring 1/12/2011 11:11'!
at: row at: column put: data
	(rows at: row) at: column put: data! !

!KmGrid methodsFor: 'accessing' stamp: 'StuartHerring 1/10/2011 11:59'!
columns
	^cols! !

!KmGrid methodsFor: 'enumerating' stamp: 'StuartHerring 1/30/2011 23:03'!
do: aBlock
	^rows do: [ :r | r do: [ :c | aBlock value: c]]! !

!KmGrid methodsFor: 'initialization' stamp: 'StuartHerring 1/10/2011 12:12'!
initialize
	super initialize.
	rows := OrderedCollection new.
	cols := 0.! !

!KmGrid methodsFor: 'actions' stamp: 'StuartHerring 1/17/2011 11:50'!
prependColumn
	rows do:  [:row |
		 row addFirst: nil ].
	cols := cols + 1.
		! !

!KmGrid methodsFor: 'actions' stamp: 'StuartHerring 1/17/2011 11:48'!
prependRow
	rows addFirst: (OrderedCollection ofSize: cols)! !

!KmGrid methodsFor: 'accessing' stamp: 'StuartHerring 1/10/2011 11:59'!
rows
	^rows size! !

!KmGrid methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 21:30'!
surroundingItemsAt: row at:   col
	^(((((1 max: row - 1) to: (self rows min: row + 1)) collect: [ :r | 
		((1 max: col - 1) to:  (self columns min: col + 1)) collect: [ :c | c@r ]]) 
			gather: [ :a | a]) 
				reject: [:a | a = (col@row)])
					collect: [ :a | self at: a y at: a x]! !

GRObject subclass: #KmLocation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

KmLocation subclass: #KmCentre
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

KmCentre subclass: #Centre
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

!Centre class methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 16:20'!
adjacentLocations
	^KmCentre all reject: [ :s | s = self]! !

!Centre class methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 16:20'!
connectingLocations
	^self adjacentLocations! !

!Centre class methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 14:39'!
edge
	self shouldNotImplement! !

!Centre class methodsFor: 'accessing' stamp: 'StuartHerring 5/16/2011 10:09'!
generateMethod: aSelector
	"Centre is a special case - don't generate anything' "  ! !

!Centre class methodsFor: 'accessing' stamp: 'StuartHerring 5/16/2011 10:11'!
opposite
	self shouldNotImplement! !

!Centre class methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 15:03'!
ordinal
	^0! !

KmCentre subclass: #CentreBottom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

!CentreBottom class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
adjacentLocations
^{Bottom.Centre.}! !

!CentreBottom class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
connectingLocations
^{Bottom.Centre.LeftBottom.RightBottom.}! !

!CentreBottom class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
edge
^Bottom! !

!CentreBottom class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
opposite
^CentreTop! !

!CentreBottom class methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 15:03'!
ordinal
	^11! !

KmCentre subclass: #CentreLeft
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

!CentreLeft class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
adjacentLocations
^{Left.Centre.}! !

!CentreLeft class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
connectingLocations
^{Left.Centre.TopLeft.BottomLeft.}! !

!CentreLeft class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
edge
^Left! !

!CentreLeft class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
opposite
^CentreRight! !

!CentreLeft class methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 15:03'!
ordinal
	^15! !

KmCentre subclass: #CentreRight
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

!CentreRight class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
adjacentLocations
^{Right.Centre.}! !

!CentreRight class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
connectingLocations
^{Right.Centre.BottomRight.TopRight.}! !

!CentreRight class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
edge
^Right! !

!CentreRight class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
opposite
^CentreLeft! !

!CentreRight class methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 15:03'!
ordinal
	^7! !

KmCentre subclass: #CentreTop
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

!CentreTop class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
adjacentLocations
^{Top.Centre.}! !

!CentreTop class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
connectingLocations
^{Top.Centre.RightTop.LeftTop.}! !

!CentreTop class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
edge
^Top! !

!CentreTop class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
opposite
^CentreBottom! !

!CentreTop class methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 15:03'!
ordinal
	^3! !

!KmCentre class methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 14:41'!
adjacentLocations
	^Array with: self edge with: Centre! !

!KmCentre class methodsFor: 'accessing' stamp: 'StuartHerring 5/16/2011 10:03'!
connectingLocations
	^self adjacentLocations asOrderedCollection addAll: {
		KmLocation all at: ((self ordinal + 2) \\ 16) + 1.
		KmLocation all at: ((self ordinal + 12) \\ 16) + 2};
	yourself! !

!KmCentre class methodsFor: 'testing' stamp: 'StuartHerring 5/14/2011 14:39'!
isCentre
	^true! !

KmLocation subclass: #KmCorner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

KmCorner subclass: #BottomLeft
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

!BottomLeft class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
adjacentLocations
^{Bottom.}! !

!BottomLeft class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
connectingLocations
^{Bottom.LeftBottom.CentreLeft.}! !

!BottomLeft class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
edge
^Bottom! !

!BottomLeft class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
opposite
^TopLeft! !

!BottomLeft class methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 15:03'!
ordinal
	^12! !

KmCorner subclass: #BottomRight
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

!BottomRight class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
adjacentLocations
^{Bottom.}! !

!BottomRight class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
connectingLocations
^{Bottom.RightBottom.CentreRight.}! !

!BottomRight class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
edge
^Bottom! !

!BottomRight class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
opposite
^TopRight! !

!BottomRight class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/14/2011 15:03'!
ordinal
	^9! !

!KmCorner class methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 14:45'!
adjacentLocations
	^Array with: self edge! !

!KmCorner class methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 16:19'!
connectingLocations
	| adj |
	adj := (self ordinal > self edge ordinal)
			ifTrue: [[ :a | KmLocation all at: (((a ordinal + 1) \\ 16) + 1)]]
			ifFalse: [[ :a | KmLocation all at: (((a ordinal + 14) \\ 16) + 2)]].
	^self adjacentLocations asOrderedCollection
		addAll:  {adj value: self. (adj value: self) edge centre};
		yourself! !

!KmCorner class methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 16:52'!
opposite
		^KmLocation all at: (
			(self ordinal < self edge ordinal) 
				ifTrue: [self edge opposite ordinal + 3]
				ifFalse: [ self edge opposite ordinal ])! !

KmCorner subclass: #LeftBottom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

!LeftBottom class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
adjacentLocations
^{Left.}! !

!LeftBottom class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
connectingLocations
^{Left.BottomLeft.CentreBottom.}! !

!LeftBottom class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
edge
^Left! !

!LeftBottom class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
opposite
^RightBottom! !

!LeftBottom class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/14/2011 15:03'!
ordinal
	^13! !

KmCorner subclass: #LeftTop
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

!LeftTop class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
adjacentLocations
^{Left.}! !

!LeftTop class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
connectingLocations
^{Left.TopLeft.CentreTop.}! !

!LeftTop class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
edge
^Left! !

!LeftTop class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
opposite
^RightTop! !

!LeftTop class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/14/2011 15:03'!
ordinal
	^16! !

KmCorner subclass: #RightBottom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

!RightBottom class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
adjacentLocations
^{Right.}! !

!RightBottom class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
connectingLocations
^{Right.BottomRight.CentreBottom.}! !

!RightBottom class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
edge
^Right! !

!RightBottom class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
opposite
^LeftBottom! !

!RightBottom class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/14/2011 15:03'!
ordinal
	^8! !

KmCorner subclass: #RightTop
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

!RightTop class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
adjacentLocations
^{Right.}! !

!RightTop class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
connectingLocations
^{Right.TopRight.CentreTop.}! !

!RightTop class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
edge
^Right! !

!RightTop class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
opposite
^LeftTop! !

!RightTop class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/14/2011 15:03'!
ordinal
	^5! !

KmCorner subclass: #TopLeft
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

!TopLeft class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
adjacentLocations
^{Top.}! !

!TopLeft class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
connectingLocations
^{Top.LeftTop.CentreLeft.}! !

!TopLeft class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
edge
^Top! !

!TopLeft class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
opposite
^BottomLeft! !

!TopLeft class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/14/2011 15:03'!
ordinal
	^1! !

KmCorner subclass: #TopRight
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

!TopRight class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
adjacentLocations
^{Top.}! !

!TopRight class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
connectingLocations
^{Top.RightTop.CentreRight.}! !

!TopRight class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
edge
^Top! !

!TopRight class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
opposite
^BottomRight! !

!TopRight class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/14/2011 15:03'!
ordinal
	^4! !

KmLocation subclass: #KmEdge
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

KmEdge subclass: #Bottom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

!Bottom class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
adjacentLocations
^{BottomRight.BottomLeft.CentreBottom.}! !

!Bottom class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
connectingLocations
^{BottomRight.BottomLeft.CentreBottom.}! !

!Bottom class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
edge
^Bottom! !

!Bottom class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
opposite
^Top! !

!Bottom class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/14/2011 15:03'!
ordinal
	^10! !

!KmEdge class methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 15:29'!
adjacentLocations
	| all |
	all := KmLocation all.
	^{all at: self ordinal. all at: self ordinal + 3. all at: self ordinal + 2}! !

!KmEdge class methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 16:17'!
centre
	^KmLocation all after: self! !

!KmEdge class methodsFor: 'accessing' stamp: 'StuartHerring 1/30/2011 21:24'!
edge
	^self! !

!KmEdge class methodsFor: 'accessing' stamp: 'StuartHerring 5/16/2011 10:04'!
locations
	| all |
	all := KmLocation all.
	^{all at: self ordinal. 
		self. 
		all at: self ordinal + 3}! !

KmEdge subclass: #Left
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

!Left class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
adjacentLocations
^{LeftBottom.LeftTop.CentreLeft.}! !

!Left class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
connectingLocations
^{LeftBottom.LeftTop.CentreLeft.}! !

!Left class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
edge
^Left! !

!Left class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
opposite
^Right! !

!Left class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/14/2011 15:03'!
ordinal
	^14! !

KmEdge subclass: #Right
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

!Right class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
adjacentLocations
^{RightTop.RightBottom.CentreRight.}! !

!Right class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
connectingLocations
^{RightTop.RightBottom.CentreRight.}! !

!Right class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
edge
^Right! !

!Right class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
opposite
^Left! !

!Right class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/14/2011 15:03'!
ordinal
	^6! !

KmEdge subclass: #Top
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Locations'!

!Top class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
adjacentLocations
^{TopLeft.TopRight.CentreTop.}! !

!Top class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
connectingLocations
^{TopLeft.TopRight.CentreTop.}! !

!Top class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
edge
^Top! !

!Top class methodsFor: 'generated' stamp: 'StuartHerring 5/16/2011 11:19'!
opposite
^Bottom! !

!Top class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/14/2011 15:03'!
ordinal
	^2! !

!KmLocation class methodsFor: 'comparing' stamp: 'StuartHerring 5/14/2011 15:21'!
<= aLeftClass 
	^self ordinal <= aLeftClass ordinal! !

!KmLocation class methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 14:33'!
adjacentLocations
	"The locations directly adjacent to this location, clockwise from the right"
	self subclassResponsibility! !

!KmLocation class methodsFor: 'grouping' stamp: 'StuartHerring 5/14/2011 14:52'!
all
	^(self allSubclasses reject: [ :c | c isAbstract ]) sorted.! !

!KmLocation class methodsFor: 'comparing' stamp: 'StuartHerring 5/16/2011 11:03'!
arrayMethodBodyFor: aCollection
	| aStream |
	aStream := WriteStream on: ''.
	aStream nextPut: ${.
	aCollection do: [ :a | 	aStream nextPutAll: a name; nextPut: $. ].
	aStream nextPut: $}.
	^aStream contents.! !

!KmLocation class methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 15:26'!
connectingLocations
	"The locations to be considered reachable from this location for the purposes of connecting a territory, starting from the right"
	^self adjacentLocations! !

!KmLocation class methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 15:51'!
edge
	^KmLocation all at: (((self ordinal - 1) / 4) asInteger * 4) + 3! !

!KmLocation class methodsFor: 'grouping' stamp: 'StuartHerring 1/12/2011 13:22'!
edges
	^{Top. Right. Bottom. Left.}! !

!KmLocation class methodsFor: 'comparing' stamp: 'StuartHerring 5/16/2011 11:19'!
generateMethod: aSelector
	| s result |
	result := self perform: aSelector.
	s := WriteStream on: ''.
	s nextPutAll: aSelector;
		nextPutAll: '
^'.
	s nextPutAll:
		 (result isCollection 
			ifTrue: [self arrayMethodBodyFor: result] 
			ifFalse: [result name]).
 	self class compile: s contents classified:  #generated! !

!KmLocation class methodsFor: 'comparing' stamp: 'StuartHerring 5/16/2011 10:16'!
generateMethods
	self all do: [ :a |
		#(edge opposite adjacentLocations connectingLocations) do: [ :m | 
				a generateMethod: m]]
	! !

!KmLocation class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/14/2011 14:52'!
isAbstract
	^name beginsWith: 'Km'! !

!KmLocation class methodsFor: 'testing' stamp: 'StuartHerring 4/22/2011 14:00'!
isCentre
	^false! !

!KmLocation class methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 15:50'!
opposite
	^KmLocation all at: (self ordinal + 8 \\ 16) + 1! !

!KmLocation class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/14/2011 14:54'!
ordinal
	^self subclassResponsibility! !

GRObject subclass: #KmPiece
	instanceVariableNames: 'player'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmPiece class methodsFor: 'instance creation' stamp: 'StuartHerring 1/7/2011 15:07'!
forPlayer: aPlayer
	^self basicNew
		initializeWithPlayer: aPlayer;
		yourself! !

!KmPiece methodsFor: 'initialization' stamp: 'StuartHerring 7/6/2011 21:36'!
colour
	^player colour! !

!KmPiece methodsFor: 'initialization' stamp: 'StuartHerring 1/7/2011 15:06'!
initializeWithPlayer: aPlayer
	self initialize.
	player := aPlayer! !

!KmPiece methodsFor: 'initialization' stamp: 'StuartHerring 5/9/2011 20:27'!
player
	^player! !

!KmPiece methodsFor: 'initialization' stamp: 'StuartHerring 5/14/2011 20:08'!
printOn: aStream
	super printOn: aStream.
	aStream 
		nextPutAll: ' (';
		nextPutAll: player name;
		nextPut: $).    ! !

GRObject subclass: #KmPlayer
	instanceVariableNames: 'pieces colour score user'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmPlayer class methodsFor: 'instance creation' stamp: 'StuartHerring 6/6/2011 21:59'!
forUser: aUser
	^self new
		setUser: aUser;
		yourself! !

!KmPlayer methodsFor: 'actions' stamp: 'StuartHerring 5/12/2011 11:04'!
addScore: aSmallInteger 
	score := score + aSmallInteger! !

!KmPlayer methodsFor: 'accessing' stamp: 'StuartHerring 6/6/2011 00:29'!
colour
	^colour ! !

!KmPlayer methodsFor: 'accessing' stamp: 'StuartHerring 6/6/2011 00:29'!
colour: aColour
	colour := aColour! !

!KmPlayer methodsFor: 'initialization' stamp: 'StuartHerring 6/6/2011 00:30'!
initialize
	super initialize.
	score := 0.
	pieces := ((1 to: 5) collect: [ :x | KmPiece forPlayer: self]) asSet.! !

!KmPlayer methodsFor: 'accessing' stamp: 'StuartHerring 6/6/2011 00:28'!
name
	^user name! !

!KmPlayer methodsFor: 'accessing' stamp: 'StuartHerring 5/9/2011 20:20'!
pieces
	^pieces! !

!KmPlayer methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/6/2011 22:02'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' (';
		nextPutAll: self name;
		nextPutAll: ')'    ! !

!KmPlayer methodsFor: 'actions' stamp: 'StuartHerring 5/9/2011 20:31'!
returnPiece: aPiece
	aPiece player = self 
		ifFalse: [ self error: 'Cannot return a piece that belongs to another player'].
	pieces add: aPiece! !

!KmPlayer methodsFor: 'accessing' stamp: 'StuartHerring 5/11/2011 07:38'!
score
	^score! !

!KmPlayer methodsFor: 'private' stamp: 'StuartHerring 6/6/2011 00:21'!
setUser: aUser
	user := aUser! !

!KmPlayer methodsFor: 'actions' stamp: 'StuartHerring 5/9/2011 20:23'!
takePiece
	^pieces remove: (pieces any)! !

!KmPlayer methodsFor: 'private' stamp: 'StuartHerring 6/22/2011 21:12'!
user
	^user.! !

GRObject subclass: #KmTerritory
	instanceVariableNames: 'segments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

KmTerritory subclass: #KmFarmTerritory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmFarmTerritory class methodsFor: 'accessing' stamp: 'StuartHerring 5/11/2011 22:14'!
codeCharacter
	^$F! !

!KmFarmTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/14/2011 17:03'!
calculateFinalScore
	^self hasPieces ifTrue: [
		self dominantPlayers do: [ :player | 
			player addScore: (self adjacentTerritories count: [ : t | t isVillage and: [t isComplete]]) * self completedScore]]! !

!KmFarmTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/14/2011 17:21'!
calculateScore
	"Farms don't get scored yet"! !

!KmFarmTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/14/2011 17:03'!
completedScore
	"For farms, this really means score per completed village"
	^4! !

!KmFarmTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/11/2011 20:10'!
isFarm
	^true! !

!KmFarmTerritory methodsFor: 'accessing' stamp: 'StuartHerring 5/11/2011 20:56'!
type
	^#farm! !

KmTerritory subclass: #KmHutTerritory
	instanceVariableNames: 'surroundingTiles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmHutTerritory class methodsFor: 'accessing' stamp: 'StuartHerring 5/11/2011 22:13'!
codeCharacter
	^$+! !

!KmHutTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/14/2011 20:55'!
addSurroundingTile
	surroundingTiles = 8 ifTrue: [KmInvalidGameState signal: 'Can''t have more than 8 tiles surrounding a hut it''s impossible'].
	surroundingTiles := surroundingTiles + 1.! !

!KmHutTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/12/2011 11:24'!
completedScore
	^9! !

!KmHutTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/14/2011 20:52'!
initialize
	super initialize.
	surroundingTiles := 0.! !

!KmHutTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/14/2011 20:55'!
isComplete
	^surroundingTiles = 8! !

!KmHutTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/11/2011 20:11'!
isHut
	^true! !

!KmHutTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/14/2011 20:53'!
surroundingTiles
	^surroundingTiles! !

!KmHutTerritory methodsFor: 'accessing' stamp: 'StuartHerring 5/11/2011 20:56'!
type
	^#hut! !

!KmHutTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/14/2011 21:54'!
uncompletedScore
	^1 + surroundingTiles! !

KmTerritory subclass: #KmNullTerritory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmNullTerritory class methodsFor: 'accessing' stamp: 'StuartHerring 5/11/2011 22:14'!
codeCharacter
	^$.! !

!KmNullTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/11/2011 23:12'!
canPlacePiece
	^false! !

!KmNullTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/11/2011 21:50'!
isComplete
	^false! !

!KmNullTerritory methodsFor: 'accessing' stamp: 'StuartHerring 5/11/2011 20:56'!
type
	^#blocker! !

KmTerritory subclass: #KmRoadTerritory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmRoadTerritory class methodsFor: 'accessing' stamp: 'StuartHerring 5/11/2011 22:14'!
codeCharacter
	^$-! !

!KmRoadTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/12/2011 11:20'!
completedScore
	^self tiles size * 2! !

!KmRoadTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/11/2011 20:11'!
isRoad
	^true! !

!KmRoadTerritory methodsFor: 'accessing' stamp: 'StuartHerring 5/11/2011 20:56'!
type
	^#road! !

!KmRoadTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/12/2011 11:20'!
uncompletedScore
	^self tiles size * 1! !

!KmTerritory class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/11/2011 22:14'!
codeCharacter
	self subclassResponsibility! !

!KmTerritory class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/11/2011 23:03'!
newSegment
	^KmTileSegment inTerritory: self new ! !

!KmTerritory class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/11/2011 23:04'!
newSegmentFromChar: aCharacter
	^(self allSubclasses detect: [ :territoryClass | territoryClass codeCharacter = aCharacter])
		newSegment! !

!KmTerritory methodsFor: 'adding' stamp: 'StuartHerring 1/30/2011 23:52'!
add: aSegment
	segments add: aSegment.
! !

!KmTerritory methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/14/2011 21:50'!
addSurroundingTile
	"Do Nothing"! !

!KmTerritory methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/14/2011 20:20'!
adjacentTerritories
	^((segments gather: [ :seg | seg adjacent])
		collect: [ :seg | seg territory ]) asSet
			reject: [ :t | t == self]! !

!KmTerritory methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/14/2011 17:18'!
calculateFinalScore
	^self hasPieces ifTrue: [
		self dominantPlayers do: [ :player | 
			player addScore: self uncompletedScore]]! !

!KmTerritory methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/14/2011 17:19'!
calculateScore
	^(self isComplete and: [self hasPieces]) ifTrue: [
		self dominantPlayers do: [ :player | 
			player addScore: self completedScore	].
		segments do: [ :segment | segment returnPiece]]! !

!KmTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/11/2011 21:20'!
canPlacePiece
	^self hasPieces not! !

!KmTerritory methodsFor: 'accessing' stamp: 'StuartHerring 5/12/2011 11:18'!
completedScore
	^0! !

!KmTerritory methodsFor: 'adding' stamp: 'StuartHerring 5/14/2011 20:35'!
dominantPlayers
	| playerCounts max |
	playerCounts := Dictionary new.
	self pieces do: [ :p | playerCounts at: p player put: (playerCounts at: p player ifAbsent: [0]) + 1].
	max := playerCounts values max.
	^(playerCounts select: [ :count | count = max ]) keys asSet! !

!KmTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/9/2011 19:15'!
hasPieces
	^segments anySatisfy:  [ :s | s hasPiece ]  ! !

!KmTerritory methodsFor: 'initialization' stamp: 'StuartHerring 5/7/2011 21:08'!
initialize
	super initialize.
	segments := Set new! !

!KmTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/7/2011 21:08'!
isComplete
	^segments noneSatisfy: [ :s | s isAtEdge]! !

!KmTerritory methodsFor: 'testing' stamp: 'StuartHerring 1/30/2011 17:55'!
isEmpty
	^self size = 0.! !

!KmTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/11/2011 19:50'!
isFarm
	^false! !

!KmTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/11/2011 19:50'!
isHut
	^false! !

!KmTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/11/2011 19:50'!
isRoad
	^false! !

!KmTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/11/2011 19:50'!
isVillage
	^false! !

!KmTerritory methodsFor: 'merging' stamp: 'StuartHerring 5/11/2011 19:36'!
mergeWith: aTerritory
	"Removes all segments from this territory and puts them in the provided one"
	aTerritory == self ifFalse: [
		segments do: [ :s | s territory: aTerritory].
		segments removeAll].! !

!KmTerritory methodsFor: 'accessing' stamp: 'StuartHerring 5/9/2011 19:11'!
pieces
	^segments select: [ :s | s hasPiece ] thenCollect: [ :s | s piece]! !

!KmTerritory methodsFor: 'accessing' stamp: 'StuartHerring 1/30/2011 16:08'!
size
	^segments size! !

!KmTerritory methodsFor: 'accessing' stamp: 'StuartHerring 1/30/2011 19:37'!
tiles
	^Set withAll: (segments collect: [:s | s tile])! !

!KmTerritory methodsFor: 'accessing' stamp: 'StuartHerring 5/11/2011 20:06'!
type
	self subclassResponsibility! !

!KmTerritory methodsFor: 'accessing' stamp: 'StuartHerring 5/12/2011 11:18'!
uncompletedScore
	^0! !

KmTerritory subclass: #KmVillageTerritory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmVillageTerritory class methodsFor: 'accessing' stamp: 'StuartHerring 5/11/2011 22:15'!
codeCharacter
	^$V! !

!KmVillageTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/12/2011 11:19'!
completedScore 
	^self tiles size * 2! !

!KmVillageTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/11/2011 20:11'!
isVillage
	^true! !

!KmVillageTerritory methodsFor: 'accessing' stamp: 'StuartHerring 5/11/2011 20:56'!
type
	^#village! !

!KmVillageTerritory methodsFor: 'testing' stamp: 'StuartHerring 5/12/2011 11:20'!
uncompletedScore 
	^self tiles size * 1! !

KmVillageTerritory subclass: #KmVillageWallTerritory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmVillageWallTerritory class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/11/2011 22:15'!
codeCharacter
	^$W! !

!KmVillageWallTerritory class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/11/2011 22:54'!
new
	^KmVillageTerritory new! !

!KmVillageWallTerritory class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/11/2011 23:03'!
newSegment
	^super newSegment
		beWall;
		yourself! !

GRObject subclass: #KmTile
	instanceVariableNames: 'regions regionMap tiles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmTile class methodsFor: 'standard tiles' stamp: 'StuartHerring 3/7/2011 19:57'!
blank
	"Creates a blank tile consisting of nothing but farm"
	^self buildFromString: 'FFFFFFFFF'.! !

!KmTile class methodsFor: 'instance creation' stamp: 'StuartHerring 4/27/2011 18:24'!
buildFromString: aString
	^self new
		buildFromStream: (ReadStream on: aString);
		yourself! !

!KmTile class methodsFor: 'standard tiles' stamp: 'StuartHerring 5/14/2011 10:50'!
startingTile
	"Creates the standard starting tile of a village with a road going past it"
	^self buildFromString: 'VFFFV-F--F-F-'.! !

!KmTile methodsFor: 'actions' stamp: 'StuartHerring 5/14/2011 21:50'!
addSurroundingTile
	(self at: Centre) territory addSurroundingTile! !

!KmTile methodsFor: 'initialization' stamp: 'StuartHerring 1/30/2011 19:02'!
allowsTile: aTile at: aSide
	(self tileAt: aSide) ifNotNil: [ ^false].
	aSide locations do: [ :r | ((self at: r) connectsTo: (aTile at: r opposite)) ifFalse: [ ^false]].
	^true! !

!KmTile methodsFor: 'accessing' stamp: 'StuartHerring 1/28/2011 20:31'!
at: aSymbol
	^regions at: (regionMap at: aSymbol)! !

!KmTile methodsFor: 'initialization' stamp: 'StuartHerring 4/27/2011 18:27'!
buildCentrePathFrom: char at:   index
	self buildSegmentFrom: char at: index * 4 - 1.! !

!KmTile methodsFor: 'initialization' stamp: 'StuartHerring 5/7/2011 15:48'!
buildFromStream: aStream
	1 to: 4 do: [ :i | self buildSideFrom: aStream next at: i ].
	1 to: 5 do: [ :i | self buildPathFrom: aStream next at: i ].
	(aStream atEnd or: [aStream peek = $| ]) ifFalse: [
		1 to: 4 do: [ :i | self buildCentrePathFrom: aStream next at: i]].
	self calculateTerritories! !

!KmTile methodsFor: 'initialization' stamp: 'StuartHerring 4/27/2011 18:27'!
buildPathFrom: char at: index
	index = 5 ifTrue: [self buildSegmentFrom: char at: 0]
		ifFalse: [
			self buildSegmentFrom: char at: index * 4 - 2.
			self buildCentrePathFrom: char at: index  ]! !

!KmTile methodsFor: 'initialization' stamp: 'StuartHerring 1/30/2011 17:45'!
buildSegmentFrom: char at: index
	(regions at: index  put: (KmTileSegment fromChar: char in: self))
		location: (regionMap keyAtValue: index)! !

!KmTile methodsFor: 'initialization' stamp: 'StuartHerring 4/27/2011 18:27'!
buildSideFrom: char at: index
	self buildSegmentFrom: char at:  index * 4 - 3.
	self buildSegmentFrom: char at:  index * 4.! !

!KmTile methodsFor: 'initialization' stamp: 'StuartHerring 1/30/2011 21:13'!
calculateTerritories
	KmLocation all do: [ :r | | seg1 | seg1 := (self at: r).
		r connectingLocations do: [ :r2 | | seg2 | seg2 := (self at: r2).
			(seg1 connectsTo: seg2) 
				ifTrue: [	seg2 territory mergeWith: seg1 territory]]].! !

!KmTile methodsFor: 'initialization' stamp: 'StuartHerring 4/20/2011 17:02'!
initialize
	super initialize.
	tiles := Dictionary new.
	regions := Dictionary new.
	regionMap := Dictionary new.
	KmLocation all
		keysAndValuesDo: [ :index :region | regionMap at: region put: index - 1 ].! !

!KmTile methodsFor: 'actions' stamp: 'StuartHerring 4/21/2011 12:59'!
rotate
	regionMap keysAndValuesDo: [ :key : value |
		value = 0 ifFalse: [ | new |
			new := ((value + 11 ) \\ 16) + 1.
			regionMap at: key put: new.
			(regions at: new) location: key]].! !

!KmTile methodsFor: 'initialization' stamp: 'StuartHerring 1/30/2011 21:14'!
territories
	^Set new 
		addAll: (regions select: [ :r | r territory notNil ] thenCollect: [ :r | r territory]);
		yourself! !

!KmTile methodsFor: 'accessing' stamp: 'StuartHerring 1/12/2011 13:10'!
tileAt: aDirection
	^tiles at: aDirection ifAbsent: [nil]! !

!KmTile methodsFor: 'accessing' stamp: 'StuartHerring 1/30/2011 22:43'!
tileAt: aDirection put: aTile
	aTile ifNil: [^nil].
	tiles at: aDirection ifAbsent: [
		tiles at: aDirection put: aTile.
		aTile tileAt: aDirection opposite put: self].
	aDirection locations do: [ :loc | (self at: loc) territory mergeWith: (aTile at: loc opposite) territory].
	^aTile! !

GRObject subclass: #KmTileSegment
	instanceVariableNames: 'territory tile location piece segmentIsWall'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmTileSegment class methodsFor: 'instance creation' stamp: 'StuartHerring 5/11/2011 23:04'!
fromChar: aChar in: aTile
	^(KmTerritory newSegmentFromChar: aChar)
			tile: aTile;
			yourself.! !

!KmTileSegment class methodsFor: 'instance creation' stamp: 'StuartHerring 5/11/2011 21:59'!
inTerritory: aTerritory 
	^self basicNew
		initializeWithTerritory: aTerritory;
		yourself  ! !

!KmTileSegment methodsFor: 'accessing' stamp: 'StuartHerring 5/14/2011 16:54'!
adjacent
	| result |
	result :=(location adjacentLocations 
		collect: [ :l | 	tile at: l]) asSet.
	(location isCentre not and: [ (tile tileAt: (location edge)) notNil]) 
		ifTrue: [ result add: ((tile tileAt: location edge) at: location opposite)].
	^result! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 5/11/2011 21:14'!
beWall
	segmentIsWall := true! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 5/11/2011 21:18'!
canPlacePiece
	^territory canPlacePiece! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 5/11/2011 21:24'!
connectsTo: aSegment
	^(self tile = aSegment tile and: [aSegment isWall and: [self isWall]])
		ifTrue: [false]
		ifFalse: [self type = aSegment type]! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 5/9/2011 19:03'!
hasPiece
	^piece notNil! !

!KmTileSegment methodsFor: 'initialization' stamp: 'StuartHerring 5/11/2011 22:58'!
initializeWithTerritory: aTerritory  
	self initialize.
	segmentIsWall := false.
	self territory: aTerritory
! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 4/22/2011 14:02'!
isAtEdge
	^location isNil or: [
		location isCentre 
			ifTrue: [false]
			ifFalse: [(tile tileAt: location edge) isNil]]! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 1/31/2011 21:21'!
isComplete
	^territory isComplete! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 5/11/2011 19:39'!
isFarm 
	^territory isFarm! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 5/11/2011 20:22'!
isHut
	^territory isHut! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 5/11/2011 19:39'!
isRoad
	^territory isRoad! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 5/11/2011 19:39'!
isVillage 
	^territory isVillage! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 5/11/2011 21:15'!
isWall
	^segmentIsWall! !

!KmTileSegment methodsFor: 'accessing' stamp: 'StuartHerring 1/30/2011 16:16'!
location
	^location! !

!KmTileSegment methodsFor: 'accessing' stamp: 'StuartHerring 1/30/2011 16:23'!
location: aLocation
	location := aLocation.! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 5/9/2011 19:04'!
piece
	^piece! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 5/9/2011 19:21'!
piece: aPiece
	self canPlacePiece ifFalse: [ self error: 'One or more pieces already exist in territory'].
	piece := aPiece! !

!KmTileSegment methodsFor: 'printing' stamp: 'StuartHerring 1/30/2011 18:20'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ', '.
	aStream nextPutAll: location name! !

!KmTileSegment methodsFor: 'initialization' stamp: 'StuartHerring 5/12/2011 11:16'!
returnPiece
	piece ifNotNil: [ :p | p player returnPiece: p ].
	piece := nil.! !

!KmTileSegment methodsFor: 'accessing' stamp: 'StuartHerring 1/30/2011 14:02'!
territory
	^territory! !

!KmTileSegment methodsFor: 'accessing' stamp: 'StuartHerring 1/30/2011 17:48'!
territory: aTerritory
	territory := aTerritory.
	aTerritory add: self.! !

!KmTileSegment methodsFor: 'accessing' stamp: 'StuartHerring 1/30/2011 16:16'!
tile
	^tile! !

!KmTileSegment methodsFor: 'accessing' stamp: 'StuartHerring 5/11/2011 21:56'!
tile: aTile
	tile := aTile! !

!KmTileSegment methodsFor: 'accessing' stamp: 'StuartHerring 5/11/2011 20:07'!
type
	^territory type! !

Error subclass: #KmInvalidGameState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

Error subclass: #KmInvalidMove
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

Object subclass: #KmRegistry
	instanceVariableNames: 'users games'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

KmRegistry class
	instanceVariableNames: 'default'!

KmRegistry class
	instanceVariableNames: 'default'!

!KmRegistry class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/12/2011 15:58'!
default
	^default ifNil: [default := KmRegistry new]! !

!KmRegistry methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/6/2011 00:31'!
addUser: aUser
	^users add: aUser! !

!KmRegistry methodsFor: 'as yet unclassified' stamp: 'StuartHerring 7/6/2011 22:37'!
createNewGameForUser: aUser fromTileSet: aTileSet
	| g |
	g := KmGame tileSet: aTileSet.
	aUser addToGame: g.
	^games add: g! !

!KmRegistry methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/12/2011 15:31'!
games
	^games! !

!KmRegistry methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/12/2011 15:35'!
gamesCompleted
	^games select: [ :g | g isComplete]! !

!KmRegistry methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/12/2011 15:32'!
gamesInProgress
	^games select: [ :g | g inProgress]! !

!KmRegistry methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/22/2011 21:31'!
gamesPending
	^games select: [ :g | g isPending]! !

!KmRegistry methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/12/2011 15:36'!
initialize
	super initialize.
	users := Set new.
	games := Set new.! !

!KmRegistry methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/6/2011 00:31'!
users
	^users! !

Object subclass: #KmTileSet
	instanceVariableNames: 'tiles name'
	classVariableNames: 'TileSets'
	poolDictionaries: ''
	category: 'Khami-Core'!

KmTileSet subclass: #KmTSTestSet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmTSTestSet class methodsFor: 'instance creation' stamp: 'StuartHerring 5/9/2011 18:44'!
new
	^self buildFromStrings: #('VFFFV-F--F-F-' 'VVFFVVFFF' 'FFFF--FF-' 'FFFF--FF-' 'FFFF-FFFF' 'FFFFF-F--' 'VFFFVF---' 'WWWWVVVVFFFFF' 'FFFVF-FVV' 'VVVFVVVFF' 'VVVVVVVVV' 'VVVFVVVFF' 'VFFFVFFFF' 'FFFFFFFFF' 'VVVFVVV-V' 'VFFFVFFFF' 'VVFFVVFFF' 'VFVFV-V--' 'VFFFV--F-' 'FFFFF-F--' 'FVFVFVFVV' 'FVFV-V-VV' 'WWFFVVFFF')! !

!KmTSTestSet class methodsFor: 'accessing' stamp: 'StuartHerring 3/21/2011 23:04'!
tilesetName
	^'Test Set'! !

!KmTileSet class methodsFor: 'instance creation' stamp: 'StuartHerring 3/21/2011 23:00'!
buildFromStrings: aCollection
	^self buildFromTiles: 
		(aCollection select: [:string | string withBlanksTrimmed notEmpty] 
			thenCollect: [ :string | KmTile buildFromString: string ]) asOrderedCollection! !

!KmTileSet class methodsFor: 'instance creation' stamp: 'StuartHerring 3/21/2011 22:59'!
buildFromTiles: aCollection
	^self basicNew
		initializeWithTiles: aCollection asOrderedCollection;
		yourself.! !

!KmTileSet class methodsFor: 'class initialization' stamp: 'StuartHerring 3/14/2011 15:31'!
initialize
	TileSets := Set new.! !

!KmTileSet class methodsFor: 'instance creation' stamp: 'StuartHerring 3/21/2011 19:03'!
new
	self subclassResponsibility! !

!KmTileSet class methodsFor: 'accessing' stamp: 'StuartHerring 3/21/2011 21:19'!
prefix
	^'KmTS'! !

!KmTileSet class methodsFor: 'registration' stamp: 'StuartHerring 3/21/2011 23:03'!
register: aName asTileSetUsingStrings: aCollection
	| tileSetClass source |
	tileSetClass := self 
		subclass: (self prefix, aName asLegalSelector) asSymbol
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Khami-Generated-Tilesets'.
	source := '' writeStream.
	source nextPutAll: 'new
	^self buildFromStrings:'.
	source nextPutAll: aCollection asArray printString.
	tileSetClass class compile: source contents.
	tileSetClass class compile: 'tilesetName
	^', aName printString.! !

!KmTileSet class methodsFor: 'registration' stamp: 'StuartHerring 3/24/2011 20:24'!
register: aName fromFileNamed: aFilePath
	^self register: aName asTileSetUsingStrings: 	
		((FileStream fileNamed: aFilePath) contents splitOn: Character lf)
	! !

!KmTileSet class methodsFor: 'registration' stamp: 'StuartHerring 5/9/2011 21:15'!
registerFromDirectory: aName
	| baseDir |
	baseDir := FileDirectory on: Smalltalk imagePath.
	^self register: aName fromFileNamed:	
		(((baseDir directoryNamed: aName)	fileNamed: 'set.txt'))! !

!KmTileSet class methodsFor: 'tile sets' stamp: 'StuartHerring 5/9/2011 21:13'!
testSet
	^KmTSTestSet new! !

!KmTileSet class methodsFor: 'tile sets' stamp: 'StuartHerring 5/12/2011 06:50'!
tileSets
	^self allSubclasses select: [ :s | s tilesetName notNil]  ! !

!KmTileSet class methodsFor: 'accessing' stamp: 'StuartHerring 5/12/2011 06:50'!
tilesetName
	^nil! !

!KmTileSet methodsFor: 'enumerating' stamp: 'StuartHerring 3/7/2011 21:28'!
do: aBlock
	^tiles do: aBlock! !

!KmTileSet methodsFor: 'initialization' stamp: 'StuartHerring 4/26/2011 15:38'!
initializeWithTiles: aCollection
	self initialize.
	tiles := OrderedCollection new
		add: aCollection first;
		addAll: aCollection copyWithoutFirst shuffled;
		yourself! !

!KmTileSet methodsFor: 'accessing' stamp: 'StuartHerring 3/21/2011 22:58'!
name
	^self class tilesetName! !

!KmTileSet methodsFor: 'accessing' stamp: 'StuartHerring 3/7/2011 20:31'!
size
	^tiles size.! !

!KmTileSet methodsFor: 'actions' stamp: 'StuartHerring 3/7/2011 20:44'!
takeTile
	^tiles removeFirst! !

Object subclass: #KmUser
	instanceVariableNames: 'name games'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmUser class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/6/2011 00:34'!
named: aName
	^self new
		setName: aName;
		yourself! !

!KmUser methodsFor: 'accessing' stamp: 'StuartHerring 6/6/2011 00:33'!
addToGame: aGame
	games add: aGame.
	aGame addPlayer: (KmPlayer forUser: self)! !

!KmUser methodsFor: 'accessing' stamp: 'StuartHerring 6/6/2011 00:29'!
games
	^games! !

!KmUser methodsFor: 'accessing' stamp: 'StuartHerring 6/11/2011 19:11'!
gamesInProgress
	^games select: [ :g | g isComplete not]! !

!KmUser methodsFor: 'accessing' stamp: 'StuartHerring 6/6/2011 00:32'!
initialize
	super initialize.
	games := Set new.! !

!KmUser methodsFor: 'accessing' stamp: 'StuartHerring 6/6/2011 00:34'!
name
	^name! !

!KmUser methodsFor: 'accessing' stamp: 'StuartHerring 7/6/2011 23:23'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' (';
		nextPutAll: (self name ifNil: ['nil']);
		nextPutAll: ')'    ! !

!KmUser methodsFor: 'accessing' stamp: 'StuartHerring 6/6/2011 00:34'!
setName: aName
	name := aName! !
KmTileSet initialize!
